@@ -1,12 +1,14 @@
-import argparse
-from argparse import Namespace
-from pathlib import Path
-from typing import Iterable
+from argparse import ArgumentParser, Namespace
+from typing import TYPE_CHECKING, Iterable
 
+from ldb.cli_utils import add_target_dir_argument
 from ldb.config import get_ldb_dir
 from ldb.core import init_quickstart
 from ldb.stage import stage
 
+if TYPE_CHECKING:
+    from argparse import _SubParsersAction
+
 
 def stage_command(options: Namespace) -> None:
     ldb_dir = get_ldb_dir()
@@ -15,14 +17,14 @@ def stage_command(options: Namespace) -> None:
     stage(
         ldb_dir,
         options.dataset,
-        options.workspace_path,
+        options.target_dir,
         options.force,
     )
 
 
 def add_parser(
-    subparsers: argparse._SubParsersAction,
-    parents: Iterable[argparse.ArgumentParser],
+    subparsers: "_SubParsersAction[ArgumentParser]",
+    parents: Iterable[ArgumentParser],
 ) -> None:
     parser = subparsers.add_parser(
         "stage",
@@ -36,15 +38,10 @@ def add_parser(
         default=False,
         help="Overwrite an unsaved dataset",
     )
+    add_target_dir_argument(parser)
     parser.add_argument(
         "dataset",
+        metavar="<dataset>",
         help="Name of the dataset to stage",
     )
-    parser.add_argument(
-        "workspace_path",
-        nargs="?",
-        type=Path,
-        default=".",
-        help="Workspace directory to stage the dataset in",
-    )
     parser.set_defaults(func=stage_command)
