@@ -1,6 +1,9 @@
+import json
 import os
 import shutil
+import subprocess
 import tempfile
+from concurrent.futures import ThreadPoolExecutor
 from dataclasses import dataclass
 from pathlib import Path
 from typing import (
@@ -9,6 +12,7 @@ from typing import (
     Mapping,
     NamedTuple,
     Optional,
+    Sequence,
     Tuple,
     Union,
     cast,
@@ -21,15 +25,20 @@ from ldb.dataset import get_annotation
 from ldb.exceptions import LDBException
 from ldb.fs.utils import FSProtocol, first_protocol
 from ldb.path import InstanceDir, WorkspacePath
+from ldb.progress import get_progressbar
 from ldb.storage import StorageLocation, get_filesystem, get_storage_locations
 from ldb.typing import JSONDecoded, JSONObject
 from ldb.utils import (
+    DATA_OBJ_ID_PREFIX,
     get_hash_path,
     json_dumps,
     load_data_file,
     write_data_file,
 )
-from ldb.workspace import collection_dir_to_object, ensure_empty_workspace
+from ldb.workspace import (
+    collection_dir_to_object,
+    ensure_path_is_empty_workspace,
+)
 
 
 class InstantiateResult(NamedTuple):
@@ -42,8 +51,10 @@ class InstantiateResult(NamedTuple):
 def instantiate(
     ldb_dir: Path,
     workspace_path: Path,
+    dest: Path,
     fmt: str = Format.BARE,
     force: bool = False,
+    apply: Sequence[str] = (),
 ) -> InstantiateResult:
     fmt = INSTANTIATE_FORMATS[fmt]
     collection = collection_dir_to_object(
@@ -51,26 +62,31 @@ def instantiate(
     )
 
     # fail fast if workspace is not empty
-    ensure_empty_workspace(workspace_path, force)
+    if dest.exists():
+        ensure_path_is_empty_workspace(dest, force)
+    dest.mkdir(exist_ok=True)
 
     tmp_dir_base = workspace_path / WorkspacePath.TMP
     tmp_dir_base.mkdir(exist_ok=True)
-    tmp_dir = Path(tempfile.mkdtemp(dir=tmp_dir_base))
+    with tempfile.TemporaryDirectory(dir=tmp_dir_base) as tmp_dir:
+        result = instantiate_collection(
+            ldb_dir,
+            collection,
+            tmp_dir,
+            fmt,
+        )
 
-    result = instantiate_collection(
-        ldb_dir,
-        collection,
-        tmp_dir,
-        fmt,
-    )
+        if apply:
+            paths = [str(ldb_dir / InstanceDir.USER_FILTERS)]
+            apply_transform(apply, tmp_dir, os.fspath(dest), paths=paths)
+        else:
+            # check again to make sure nothing was added while writing to the
+            # temporary location
+            ensure_path_is_empty_workspace(dest, force)
+            dest_str = os.fspath(dest)
+            for path in Path(tmp_dir).iterdir():
+                shutil.move(os.fspath(path), dest_str)
 
-    # check again to make sure nothing was added while writing to the
-    # temporary location
-    ensure_empty_workspace(workspace_path, force)
-    for path in tmp_dir.iterdir():
-        shutil.move(os.fspath(path), os.fspath(workspace_path))
-
-    tmp_dir.rmdir()
     return result
 
 
@@ -109,6 +125,33 @@ def instantiate_collection(
     raise ValueError(f"Not a valid indexing format: {fmt}")
 
 
+def apply_transform(
+    proc_args: Sequence[str],
+    input_dir: str,
+    output_dir: str,
+    paths: Sequence[str] = (),
+) -> int:
+    from ldb.pipe import open_plugin  # pylint: disable=import-outside-toplevel
+
+    data = json.dumps(
+        [
+            os.path.abspath(input_dir),
+            os.path.abspath(output_dir),
+        ],
+    )
+    with open_plugin(proc_args, paths, set_cwd=True) as proc:
+        stdout, stderr = proc.communicate(data)
+        retcode = proc.poll() or 0
+        if retcode:
+            raise subprocess.CalledProcessError(
+                retcode,
+                proc.args,
+                output=stdout,
+                stderr=stderr,
+            )
+    return retcode
+
+
 @dataclass
 class InstItem:
     ldb_dir: Path
@@ -147,13 +190,16 @@ class InstItem:
     def data_object_dest(self) -> str:
         return self.base_dest + self.ext.lower()
 
-    def copy_data_object(self) -> str:
+    def copy_data_object(
+        self,
+    ) -> str:
         fs_protocol: FSProtocol = self.data_object_meta["fs"]["protocol"]
         protocol: str = first_protocol(fs_protocol)
         path: str = self.data_object_meta["fs"]["path"]
         fs = get_filesystem(path, protocol, self.storage_locations)
         os.makedirs(os.path.split(self.data_object_dest)[0], exist_ok=True)
         dest = self.data_object_dest
+
         fs.get_file(path, dest)
         return dest
 
@@ -274,8 +320,8 @@ def copy_pairs(
     dest_dir: Union[str, Path],
     strict: bool = False,
 ) -> InstantiateResult:
-    items = []
-    data_obj_paths = []
+    items: List[PairInstItem] = []
+    data_obj_paths: List[str] = []
     annot_paths = []
     num_annotations = 0
     # annotations are small and stored in ldb; copy them first
@@ -297,8 +343,18 @@ def copy_pairs(
         else:
             annot_paths.append("")
         items.append(item)
-    for item in items:
-        data_obj_paths.append(item.copy_data_object())
+
+    with ThreadPoolExecutor(max_workers=4 * (os.cpu_count() or 1)) as pool:
+        with get_progressbar(transient=True) as progress:
+            task = progress.add_task("Instantiate", total=len(items))
+
+            def worker(item: PairInstItem) -> str:
+                result = item.copy_data_object()
+                progress.update(task, advance=1)
+                return result
+
+            data_obj_paths = list(pool.map(worker, items))
+
     return InstantiateResult(
         data_obj_paths,
         annot_paths,
@@ -341,7 +397,8 @@ def copy_infer(
             raise LDBException(
                 "For tensorflow-inferred instantiate format, "
                 "all data objects must have an annotation. "
-                f"Missing annotation for data object: 0x{data_object_hash}",
+                "Missing annotation for data object: "
+                f"{DATA_OBJ_ID_PREFIX}{data_object_hash}",
             )
     data_obj_paths = []
     for data_object_hash, annotation_hash in cast(
@@ -377,7 +434,8 @@ def copy_label_studio(
             raise LDBException(
                 "For label-studio instantiate format, "
                 "all data objects must have an annotation. "
-                f"Missing annotation for data object: 0x{data_object_hash}",
+                "Missing annotation for data object: "
+                f"{DATA_OBJ_ID_PREFIX}{data_object_hash}",
             )
         annot = get_annotation(ldb_dir, annotation_hash)
         try:
@@ -387,7 +445,7 @@ def copy_label_studio(
                 "For label-studio instantiate format, "
                 f'annotations must have the key "data.{url_key}." '
                 "Malformatted annotation for data object: "
-                f"0x{data_object_hash}",
+                f"{DATA_OBJ_ID_PREFIX}{data_object_hash}",
             ) from exc
         annotations.append(annot)  # type: ignore[arg-type]
     path = LabelStudioInstItem(
